using System;
using System.Collections;
using AppstockSDK.Api;
using UnityEngine;

#nullable enable

namespace AppstockSDK.Demo.Banner
{
    [RequireComponent(typeof(SdkInitializer))]
    public class BannerDemo : MonoBehaviour
    {
        [SerializeField] private bool showVideoAd;
        public AnchoredAdPosition bannerPosition;

        private IBannerAd? _adUnit;

        #region UnityEvents

        private IEnumerator Start()
        {
            // Wait for `SdkInitializer` to init SDK.
            yield return new WaitForSeconds(1);
            Debug.Log($"[{DateTime.Now:O}] Attempting to Load Ad...");
            _adUnit = showVideoAd
                ? new BannerAd(new(320, 250))
                {
                    PlacementID = "8",
                    AdUnitFormat = AdFormat.Video,
                    AnchoredPosition = bannerPosition,
                    AdPosition = bannerPosition.ToAdPosition(),
                }
                : new BannerAd(new(320, 250))
                {
                    PlacementID = "4",
                    AnchoredPosition = bannerPosition,
                    AdPosition = bannerPosition.ToAdPosition(),
                };
            SubscribeToEvents(_adUnit);
            _adUnit.LoadAd();
        }

        #endregion

        #region PublicAPI

        public void SetHidden(bool hidden)
        {
            if (_adUnit is null)
            {
                return;
            }
            if (hidden)
            {
                _adUnit.Hide();
            }
            else
            {
                _adUnit.Show();
            }
        }

        public void DropAdUnit()
        {
            if (_adUnit is null)
            {
                return;
            }
            UnsubscribeFromEvents(_adUnit);
            _adUnit.Dispose();
            _adUnit = null;
        }

        #endregion

        #region Callbacks

        private void SubscribeToEvents(IBannerAd adUnit)
        {
            adUnit.OnAdLoaded += OnAdUnitLoaded;
            adUnit.OnAdFailed += OnAdUnitFailed;
            adUnit.OnAdClicked += OnAdUnitClicked;
            adUnit.OnAdClosed += OnAdUnitClosed;
        }

        private void UnsubscribeFromEvents(IBannerAd adUnit)
        {
            adUnit.OnAdLoaded -= OnAdUnitLoaded;
            adUnit.OnAdFailed -= OnAdUnitFailed;
            adUnit.OnAdClicked -= OnAdUnitClicked;
            adUnit.OnAdClosed -= OnAdUnitClosed;
        }

        private void OnAdUnitLoaded()
        {
            Debug.Log($"[{DateTime.Now:O}] (video: {showVideoAd}) Ad Loaded.");
        }

        private void OnAdUnitFailed(AdError? adError)
        {
            Debug.LogError($"[{DateTime.Now:O}] (video: {showVideoAd}) Ad Failed: {adError?.Message}.");
        }

        private void OnAdUnitClicked() => Debug.Log($"[{DateTime.Now:O}] (video: {showVideoAd}) Ad Clicked.");

        private void OnAdUnitClosed()
        {
            Debug.Log($"[{DateTime.Now:O}] (video: {showVideoAd}) Ad Closed.");
        }

        #endregion
    }
}
